=== modified file 'NEWS'
--- NEWS	2007-06-18 04:56:58 +0000
+++ NEWS	2007-06-18 20:10:35 +0000
@@ -19,6 +19,9 @@
     * The internal ``weave-list`` command has become ``versionedfile-list``,
       and now lists knits as well as weaves.  (Aaron Bentley)
 
+    * Automatic merge base selection uses a faster algorithm that chooses
+      better bases in criss-cross merge situations (Aaron Bentley)
+
   TESTING:
 
     * Removed the ``--keep-output`` option from selftest and clean up test

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2007-06-18 02:45:52 +0000
+++ bzrlib/builtins.py	2007-06-18 20:04:16 +0000
@@ -55,7 +55,6 @@
 from bzrlib.branch import Branch
 from bzrlib.bundle.apply_bundle import install_bundle, merge_bundle
 from bzrlib.conflicts import ConflictList
-from bzrlib.revision import common_ancestor
 from bzrlib.revisionspec import RevisionSpec
 from bzrlib.workingtree import WorkingTree
 """)
@@ -2560,10 +2559,8 @@
         last1 = branch1.last_revision()
         last2 = branch2.last_revision()
 
-        source = MultipleRevisionSources(branch1.repository, 
-                                         branch2.repository)
-        
-        base_rev_id = common_ancestor(last1, last2, source)
+        graph = branch1.repository.get_graph(branch2.repository)
+        base_rev_id = graph.find_unique_lca(last1, last2)
 
         print 'merge base is revision %s' % base_rev_id
 
@@ -2807,8 +2804,8 @@
                                              " merges.  Not cherrypicking or"
                                              " multi-merges.")
             repository = tree.branch.repository
-            base_revision = common_ancestor(parents[0],
-                                            parents[1], repository)
+            graph = repository.get_graph()
+            base_revision = graph.find_unique_lca(parents[0], parents[1])
             base_tree = repository.revision_tree(base_revision)
             other_tree = repository.revision_tree(parents[1])
             interesting_ids = None

=== modified file 'bzrlib/bundle/commands.py'
--- bzrlib/bundle/commands.py	2007-04-05 09:35:26 +0000
+++ bzrlib/bundle/commands.py	2007-06-08 21:48:42 +0000
@@ -29,7 +29,6 @@
     errors,
     urlutils,
     )
-from bzrlib.revision import common_ancestor
 """)
 
 from bzrlib.commands import Command
@@ -155,10 +154,9 @@
                                                  ' to be specified.')
             target_branch.repository.fetch(base_branch.repository, 
                                            base_branch.last_revision())
-            base_revision = common_ancestor(base_branch.last_revision(),
-                                            target_revision,
-                                            target_branch.repository)
-
+            graph = target_branch.repository.get_graph()
+            base_revision = graph.find_unique_lca(
+                base_branch.last_revision(), target_revision)
 
         if output is not None:
             fileobj = file(output, 'wb')

=== renamed file 'bzrlib/graph.py' => 'bzrlib/deprecated_graph.py'
=== added file 'bzrlib/graph.py'
--- bzrlib/graph.py	1970-01-01 00:00:00 +0000
+++ bzrlib/graph.py	2007-06-18 20:02:18 +0000
@@ -0,0 +1,340 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from bzrlib.deprecated_graph import (node_distances, select_farthest)
+from bzrlib.revision import NULL_REVISION
+
+# DIAGRAM of terminology
+#       A
+#       /\
+#      B  C
+#      |  |\
+#      D  E F
+#      |\/| |
+#      |/\|/
+#      G  H
+#
+# In this diagram, relative to G and H:
+# A, B, C, D, E are common ancestors.
+# C, D and E are border ancestors, because each has a non-common descendant.
+# D and E are least common ancestors because none of their descendants are
+# common ancestors.
+# C is not a least common ancestor because its descendant, E, is a common
+# ancestor.
+#
+# The find_unique_lca algorithm will pick A in two steps:
+# 1. find_lca('G', 'H') => ['D', 'E']
+# 2. Since len(['D', 'E']) > 1, find_lca('D', 'E') => ['A']
+
+
+
+class _StackedParentsProvider(object):
+
+    def __init__(self, parent_providers):
+        self._parent_providers = parent_providers
+
+    def get_parents(self, revision_ids):
+        """Find revision ids of the parents of a list of revisions
+
+        A list is returned of the same length as the input.  Each entry
+        is a list of parent ids for the corresponding input revision.
+
+        [NULL_REVISION] is used as the parent of the first user-committed
+        revision.  Its parent list is empty.
+
+        If the revision is not present (i.e. a ghost), None is used in place
+        of the list of parents.
+        """
+        found = {}
+        for parents_provider in self._parent_providers:
+            pending_revisions = [r for r in revision_ids if r not in found]
+            parent_list = parents_provider.get_parents(pending_revisions)
+            new_found = dict((k, v) for k, v in zip(pending_revisions,
+                             parent_list) if v is not None)
+            found.update(new_found)
+            if len(found) == len(revision_ids):
+                break
+        return [found.get(r, None) for r in revision_ids]
+
+
+class Graph(object):
+    """Provide incremental access to revision graphs.
+
+    This is the generic implementation; it is intended to be subclassed to
+    specialize it for other repository types.
+    """
+
+    def __init__(self, parents_provider):
+        """Construct a Graph that uses several graphs as its input
+
+        This should not normally be invoked directly, because there may be
+        specialized implementations for particular repository types.  See
+        Repository.get_graph()
+
+        :param parents_func: an object providing a get_parents call
+            conforming to the behavior of StackedParentsProvider.get_parents
+        """
+        self.get_parents = parents_provider.get_parents
+
+    def find_lca(self, *revisions):
+        """Determine the lowest common ancestors of the provided revisions
+
+        A lowest common ancestor is a common ancestor none of whose
+        descendants are common ancestors.  In graphs, unlike trees, there may
+        be multiple lowest common ancestors.
+
+        This algorithm has two phases.  Phase 1 identifies border ancestors,
+        and phase 2 filters border ancestors to determine lowest common
+        ancestors.
+
+        In phase 1, border ancestors are identified, using a breadth-first
+        search starting at the bottom of the graph.  Searches are stopped
+        whenever a node or one of its descendants is determined to be common
+
+        In phase 2, the border ancestors are filtered to find the least
+        common ancestors.  This is done by searching the ancestries of each
+        border ancestor.
+
+        Phase 2 is perfomed on the principle that a border ancestor that is
+        not an ancestor of any other border ancestor is a least common
+        ancestor.
+
+        Searches are stopped when they find a node that is determined to be a
+        common ancestor of all border ancestors, because this shows that it
+        cannot be a descendant of any border ancestor.
+
+        The scaling of this operation should be proportional to
+        1. The number of uncommon ancestors
+        2. The number of border ancestors
+        3. The length of the shortest path between a border ancestor and an
+           ancestor of all border ancestors.
+        """
+        border_common, common, sides = self._find_border_ancestors(revisions)
+        return self._filter_candidate_lca(border_common)
+
+    def find_difference(self, left_revision, right_revision):
+        """Determine the graph difference between two revisions"""
+        border, common, (left, right) = self._find_border_ancestors(
+            [left_revision, right_revision])
+        return (left.difference(right).difference(common),
+                right.difference(left).difference(common))
+
+    def _make_breadth_first_searcher(self, revisions):
+        return _BreadthFirstSearcher(revisions, self)
+
+    def _find_border_ancestors(self, revisions):
+        """Find common ancestors with at least one uncommon descendant.
+
+        Border ancestors are identified using a breadth-first
+        search starting at the bottom of the graph.  Searches are stopped
+        whenever a node or one of its descendants is determined to be common.
+
+        This will scale with the number of uncommon ancestors.
+
+        As well as the border ancestors, a set of seen common ancestors and a
+        list of sets of seen ancestors for each input revision is returned.
+        This allows calculation of graph difference from the results of this
+        operation.
+        """
+        common_searcher = self._make_breadth_first_searcher([])
+        common_ancestors = set()
+        searchers = [self._make_breadth_first_searcher([r])
+                     for r in revisions]
+        active_searchers = searchers[:]
+        border_ancestors = set()
+        def update_common(searcher, revisions):
+            w_seen_ancestors = searcher.find_seen_ancestors(
+                revision)
+            stopped = searcher.stop_searching_any(w_seen_ancestors)
+            common_ancestors.update(w_seen_ancestors)
+            common_searcher.start_searching(stopped)
+
+        while True:
+            if len(active_searchers) == 0:
+                return border_ancestors, common_ancestors, [s.seen for s in
+                                                            searchers]
+            try:
+                new_common = common_searcher.next()
+                common_ancestors.update(new_common)
+            except StopIteration:
+                pass
+            else:
+                for searcher in active_searchers:
+                    for revision in new_common.intersection(searcher.seen):
+                        update_common(searcher, revision)
+
+            newly_seen = set()
+            new_active_searchers = []
+            for searcher in active_searchers:
+                try:
+                    newly_seen.update(searcher.next())
+                except StopIteration:
+                    pass
+                else:
+                    new_active_searchers.append(searcher)
+            active_searchers = new_active_searchers
+            for revision in newly_seen:
+                if revision in common_ancestors:
+                    for searcher in searchers:
+                        update_common(searcher, revision)
+                    continue
+                for searcher in searchers:
+                    if revision not in searcher.seen:
+                        break
+                else:
+                    border_ancestors.add(revision)
+                    for searcher in searchers:
+                        update_common(searcher, revision)
+
+    def _filter_candidate_lca(self, candidate_lca):
+        """Remove candidates which are ancestors of other candidates.
+
+        This is done by searching the ancestries of each border ancestor.  It
+        is perfomed on the principle that a border ancestor that is not an
+        ancestor of any other border ancestor is a lowest common ancestor.
+
+        Searches are stopped when they find a node that is determined to be a
+        common ancestor of all border ancestors, because this shows that it
+        cannot be a descendant of any border ancestor.
+
+        This will scale with the number of candidate ancestors and the length
+        of the shortest path from a candidate to an ancestor common to all
+        candidates.
+        """
+        searchers = dict((c, self._make_breadth_first_searcher([c]))
+                          for c in candidate_lca)
+        active_searchers = dict(searchers)
+        # skip over the actual candidate for each searcher
+        for searcher in active_searchers.itervalues():
+            searcher.next()
+        while len(active_searchers) > 0:
+            for candidate, searcher in list(active_searchers.iteritems()):
+                try:
+                    ancestors = searcher.next()
+                except StopIteration:
+                    del active_searchers[candidate]
+                    continue
+                for ancestor in ancestors:
+                    if ancestor in candidate_lca:
+                        candidate_lca.remove(ancestor)
+                        del searchers[ancestor]
+                        if ancestor in active_searchers:
+                            del active_searchers[ancestor]
+                    for searcher in searchers.itervalues():
+                        if ancestor not in searcher.seen:
+                            break
+                    else:
+                        # if this revision was seen by all searchers, then it
+                        # is a descendant of all candidates, so we can stop
+                        # searching it, and any seen ancestors
+                        for searcher in searchers.itervalues():
+                            seen_ancestors =\
+                                searcher.find_seen_ancestors(ancestor)
+                            searcher.stop_searching_any(seen_ancestors)
+        return candidate_lca
+
+    def find_unique_lca(self, left_revision, right_revision):
+        """Find a unique LCA.
+
+        Find lowest common ancestors.  If there is no unique  common
+        ancestor, find the lowest common ancestors of those ancestors.
+
+        Iteration stops when a unique lowest common ancestor is found.
+        The graph origin is necessarily a unique lowest common ancestor.
+
+        Note that None is not an acceptable substitute for NULL_REVISION.
+        in the input for this method.
+        """
+        revisions = [left_revision, right_revision]
+        while True:
+            lca = self.find_lca(*revisions)
+            if len(lca) == 1:
+                return lca.pop()
+            revisions = lca
+
+
+class _BreadthFirstSearcher(object):
+    """Parallel search the breadth-first the ancestry of revisions.
+
+    This class implements the iterator protocol, but additionally
+    1. provides a set of seen ancestors, and
+    2. allows some ancestries to be unsearched, via stop_searching_any
+    """
+
+    def __init__(self, revisions, parents_provider):
+        self._start = set(revisions)
+        self._search_revisions = None
+        self.seen = set(revisions)
+        self._parents_provider = parents_provider 
+
+    def __repr__(self):
+        return ('_BreadthFirstSearcher(self._search_revisions=%r,'
+                ' self.seen=%r)' % (self._search_revisions, self.seen))
+
+    def next(self):
+        """Return the next ancestors of this revision.
+
+        Ancestors are returned in the order they are seen in a breadth-first
+        traversal.  No ancestor will be returned more than once.
+        """
+        if self._search_revisions is None:
+            self._search_revisions = self._start
+        else:
+            new_search_revisions = set()
+            for parents in self._parents_provider.get_parents(
+                self._search_revisions):
+                if parents is None:
+                    continue
+                new_search_revisions.update(p for p in parents if
+                                            p not in self.seen)
+            self._search_revisions = new_search_revisions
+        if len(self._search_revisions) == 0:
+            raise StopIteration()
+        self.seen.update(self._search_revisions)
+        return self._search_revisions
+
+    def __iter__(self):
+        return self
+
+    def find_seen_ancestors(self, revision):
+        """Find ancestors of this revision that have already been seen."""
+        searcher = _BreadthFirstSearcher([revision], self._parents_provider)
+        seen_ancestors = set()
+        for ancestors in searcher:
+            for ancestor in ancestors:
+                if ancestor not in self.seen:
+                    searcher.stop_searching_any([ancestor])
+                else:
+                    seen_ancestors.add(ancestor)
+        return seen_ancestors
+
+    def stop_searching_any(self, revisions):
+        """
+        Remove any of the specified revisions from the search list.
+
+        None of the specified revisions are required to be present in the
+        search list.  In this case, the call is a no-op.
+        """
+        stopped = self._search_revisions.intersection(revisions)
+        self._search_revisions = self._search_revisions.difference(revisions)
+        return stopped
+
+    def start_searching(self, revisions):
+        if self._search_revisions is None:
+            self._start = set(revisions)
+        else:
+            self._search_revisions.update(revisions.difference(self.seen))
+        self.seen.update(revisions)

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2007-06-06 01:07:25 +0000
+++ bzrlib/merge.py	2007-06-08 21:48:42 +0000
@@ -41,7 +41,7 @@
 from bzrlib.merge3 import Merge3
 from bzrlib.osutils import rename, pathjoin
 from progress import DummyProgress, ProgressPhase
-from bzrlib.revision import common_ancestor, is_ancestor, NULL_REVISION
+from bzrlib.revision import (is_ancestor, NULL_REVISION, ensure_null)
 from bzrlib.textfile import check_text_lines
 from bzrlib.trace import mutter, warning, note
 from bzrlib.transform import (TreeTransform, resolve_conflicts, cook_conflicts,
@@ -258,9 +258,15 @@
                 pb = ui.ui_factory.nested_progress_bar()
                 try:
                     this_repo = self.this_branch.repository
-                    self.base_rev_id = common_ancestor(self.this_basis, 
-                                                       self.other_basis, 
-                                                       this_repo, pb)
+                    graph = this_repo.get_graph()
+                    revisions = [ensure_null(self.this_basis),
+                                 ensure_null(self.other_basis)]
+                    if NULL_REVISION in revisions:
+                        self.base_rev_id = NULL_REVISION
+                    else:
+                        self.base_rev_id = graph.find_unique_lca(*revisions)
+                        if self.base_rev_id == NULL_REVISION:
+                            raise UnrelatedBranches()
                 finally:
                     pb.finished()
             except NoCommonAncestor:

=== modified file 'bzrlib/merge_directive.py'
--- bzrlib/merge_directive.py	2007-04-01 04:06:14 +0000
+++ bzrlib/merge_directive.py	2007-06-08 21:48:42 +0000
@@ -211,9 +211,9 @@
         else:
             submit_revision_id = submit_branch.last_revision()
             repository.fetch(submit_branch.repository, submit_revision_id)
-            ancestor_id = _mod_revision.common_ancestor(revision_id,
-                                                        submit_revision_id,
-                                                        repository)
+            graph = repository.get_graph()
+            ancestor_id = graph.find_unique_lca(revision_id,
+                                                submit_revision_id)
             type_handler = {'bundle': klass._generate_bundle,
                             'diff': klass._generate_diff,
                             None: lambda x, y, z: None }

=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2007-05-07 12:03:14 +0000
+++ bzrlib/remote.py	2007-06-08 21:48:42 +0000
@@ -19,7 +19,12 @@
 
 from cStringIO import StringIO
 
-from bzrlib import branch, errors, lockdir, repository
+from bzrlib import (
+    branch,
+    errors,
+    lockdir,
+    repository,
+)
 from bzrlib.branch import Branch, BranchReferenceFormat
 from bzrlib.bzrdir import BzrDir, RemoteBzrDirFormat
 from bzrlib.config import BranchConfig, TreeConfig
@@ -294,6 +299,10 @@
         assert response[0] in ('yes', 'no'), 'unexpected response code %s' % (response,)
         return response[0] == 'yes'
 
+    def get_graph(self, other_repository=None):
+        """Return the graph for this repository format"""
+        return self._real_repository.get_graph(other_repository)
+
     def gather_stats(self, revid=None, committers=None):
         """See Repository.gather_stats()."""
         path = self.bzrdir._path_for_remote_call(self._client)

=== modified file 'bzrlib/repofmt/knitrepo.py'
--- bzrlib/repofmt/knitrepo.py	2007-04-01 06:19:16 +0000
+++ bzrlib/repofmt/knitrepo.py	2007-06-08 21:48:42 +0000
@@ -16,8 +16,8 @@
 
 from bzrlib import (
     bzrdir,
+    deprecated_graph,
     errors,
-    graph,
     knit,
     lockable_files,
     lockdir,
@@ -39,6 +39,28 @@
 from bzrlib.trace import mutter, note, warning
 
 
+class _KnitParentsProvider(object):
+
+    def __init__(self, knit):
+        self._knit = knit
+
+    def get_parents(self, revision_ids):
+        parents_list = []
+        for revision_id in revision_ids:
+            if revision_id == _mod_revision.NULL_REVISION:
+                parents = []
+            else:
+                try:
+                    parents = self._knit.get_parents_with_ghosts(revision_id)
+                except errors.RevisionNotPresent:
+                    parents = None
+                else:
+                    if len(parents) == 0:
+                        parents = [_mod_revision.NULL_REVISION]
+            parents_list.append(parents)
+        return parents_list
+
+
 class KnitRepository(MetaDirRepository):
     """Knit format repository."""
 
@@ -144,7 +166,7 @@
         :param revision_ids: an iterable of revisions to graph or None for all.
         :return: a Graph object with the graph reachable from revision_ids.
         """
-        result = graph.Graph()
+        result = deprecated_graph.Graph()
         vf = self._get_revision_vf()
         versions = set(vf.versions())
         if not revision_ids:
@@ -202,6 +224,9 @@
         revision_id = osutils.safe_revision_id(revision_id)
         return self._get_revision_vf().get_parents(revision_id)
 
+    def _make_parents_provider(self):
+        return _KnitParentsProvider(self._get_revision_vf())
+
 
 class KnitRepository3(KnitRepository):
 

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2007-04-23 09:52:50 +0000
+++ bzrlib/repository.py	2007-06-09 01:13:31 +0000
@@ -25,6 +25,7 @@
 from bzrlib import (
     bzrdir,
     check,
+    deprecated_graph,
     errors,
     generate_ids,
     gpg,
@@ -703,7 +704,7 @@
         :param revision_ids: an iterable of revisions to graph or None for all.
         :return: a Graph object with the graph reachable from revision_ids.
         """
-        result = graph.Graph()
+        result = deprecated_graph.Graph()
         if not revision_ids:
             pending = set(self.all_revision_ids())
             required = set([])
@@ -862,6 +863,36 @@
         revision_id = osutils.safe_revision_id(revision_id)
         return self.get_inventory_weave().parent_names(revision_id)
 
+    def get_parents(self, revision_ids):
+        """See StackedParentsProvider.get_parents"""
+        parents_list = []
+        for revision_id in revision_ids:
+            if revision_id == _mod_revision.NULL_REVISION:
+                parents = []
+            else:
+                try:
+                    parents = self.get_revision(revision_id).parent_ids
+                except errors.NoSuchRevision:
+                    parents = None
+                else:
+                    if len(parents) == 0:
+                        parents = [_mod_revision.NULL_REVISION]
+            parents_list.append(parents)
+        return parents_list
+
+    def _make_parents_provider(self):
+        return self
+
+    def get_graph(self, other_repository=None):
+        """Return the graph walker for this repository format"""
+        parents_provider = self._make_parents_provider()
+        if (other_repository is not None and
+            other_repository.bzrdir.transport.base !=
+            self.bzrdir.transport.base):
+            parents_provider = graph._StackedParentsProvider(
+                [parents_provider, other_repository._make_parents_provider()])
+        return graph.Graph(parents_provider)
+
     @needs_write_lock
     def set_make_working_trees(self, new_value):
         """Set the policy flag for making working trees when creating branches.

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2007-04-10 18:30:32 +0000
+++ bzrlib/revision.py	2007-06-08 21:48:42 +0000
@@ -19,7 +19,12 @@
 
 
 import bzrlib.errors as errors
-from bzrlib.graph import node_distances, select_farthest, all_descendants, Graph
+from bzrlib.deprecated_graph import (
+    all_descendants,
+    Graph,
+    node_distances,
+    select_farthest,
+    )
 from bzrlib.osutils import contains_whitespace
 from bzrlib.progress import DummyProgress
 from bzrlib.symbol_versioning import (deprecated_function,
@@ -476,3 +481,10 @@
     """Raise ReservedId if the supplied revision_id is reserved"""
     if is_reserved_id(revision_id):
         raise errors.ReservedId(revision_id)
+
+def ensure_null(revision_id):
+    """Ensure only NULL_REVISION is used to represent the null revisionn"""
+    if revision_id is None:
+        return NULL_REVISION
+    else:
+        return revision_id

=== modified file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	2007-04-17 21:26:08 +0000
+++ bzrlib/revisionspec.py	2007-06-08 21:48:42 +0000
@@ -634,8 +634,15 @@
                 raise errors.NoCommits(b)
         revision_source = revision.MultipleRevisionSources(
                 branch.repository, other_branch.repository)
-        rev_id = revision.common_ancestor(revision_a, revision_b,
-                                          revision_source)
+        graph = branch.repository.get_graph(other_branch.repository)
+        revision_a = revision.ensure_null(revision_a)
+        revision_b = revision.ensure_null(revision_b)
+        if revision.NULL_REVISION in (revision_a, revision_b):
+            rev_id = revision.NULL_REVISION
+        else:
+            rev_id = graph.find_unique_lca(revision_a, revision_b)
+            if rev_id == revision.NULL_REVISION:
+                raise errors.NoCommonAncestor(revision_a, revision_b)
         try:
             revno = branch.revision_id_to_revno(rev_id)
         except errors.NoSuchRevision:

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2007-06-15 07:01:24 +0000
+++ bzrlib/tests/__init__.py	2007-06-18 20:04:16 +0000
@@ -2238,6 +2238,7 @@
                    'bzrlib.tests.test_counted_lock',
                    'bzrlib.tests.test_decorators',
                    'bzrlib.tests.test_delta',
+                   'bzrlib.tests.test_deprecated_graph',
                    'bzrlib.tests.test_diff',
                    'bzrlib.tests.test_dirstate',
                    'bzrlib.tests.test_errors',

=== renamed file 'bzrlib/tests/test_graph.py' => 'bzrlib/tests/test_deprecated_graph.py'
--- bzrlib/tests/test_graph.py	2006-10-11 23:08:27 +0000
+++ bzrlib/tests/test_deprecated_graph.py	2007-06-08 21:48:42 +0000
@@ -15,7 +15,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 from bzrlib.tests import TestCase
-from bzrlib.graph import node_distances, nodes_by_distance, Graph
+from bzrlib.deprecated_graph import node_distances, nodes_by_distance, Graph
 
 
 class TestBase(TestCase):

=== added file 'bzrlib/tests/test_graph.py'
--- bzrlib/tests/test_graph.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_graph.py	2007-06-18 20:02:18 +0000
@@ -0,0 +1,278 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from bzrlib import graph
+from bzrlib.revision import NULL_REVISION
+from bzrlib.tests import TestCaseWithMemoryTransport
+
+
+# Ancestry 1:
+#
+#  NULL_REVISION
+#       |
+#     rev1
+#      /\
+#  rev2a rev2b
+#     |    |
+#   rev3  /
+#     |  /
+#   rev4
+ancestry_1 = {'rev1': [NULL_REVISION], 'rev2a': ['rev1'], 'rev2b': ['rev1'],
+              'rev3': ['rev2a'], 'rev4': ['rev3', 'rev2b']}
+
+
+# Ancestry 2:
+#
+#  NULL_REVISION
+#    /    \
+# rev1a  rev1b
+#   |
+# rev2a
+#   |
+# rev3a
+#   |
+# rev4a
+ancestry_2 = {'rev1a': [NULL_REVISION], 'rev2a': ['rev1a'],
+              'rev1b': [NULL_REVISION], 'rev3a': ['rev2a'], 'rev4a': ['rev3a']}
+
+
+# Criss cross ancestry
+#
+#     NULL_REVISION
+#         |
+#        rev1
+#        /  \
+#    rev2a  rev2b
+#       |\  /|
+#       |  X |
+#       |/  \|
+#    rev3a  rev3b
+criss_cross = {'rev1': [NULL_REVISION], 'rev2a': ['rev1'], 'rev2b': ['rev1'],
+               'rev3a': ['rev2a', 'rev2b'], 'rev3b': ['rev2b', 'rev2a']}
+
+
+# Criss-cross 2
+#
+#  NULL_REVISION
+#    /   \
+# rev1a  rev1b
+#   |\   /|
+#   | \ / |
+#   |  X  |
+#   | / \ |
+#   |/   \|
+# rev2a  rev2b
+criss_cross2 = {'rev1a': [NULL_REVISION], 'rev1b': [NULL_REVISION],
+                'rev2a': ['rev1a', 'rev1b'], 'rev2b': ['rev1b', 'rev1a']}
+
+
+# Mainline:
+#
+#  NULL_REVISION
+#       |
+#      rev1
+#      /  \
+#      | rev2b
+#      |  /
+#     rev2a
+mainline = {'rev1': [NULL_REVISION], 'rev2a': ['rev1', 'rev2b'],
+            'rev2b': ['rev1']}
+
+
+# feature branch:
+#
+#  NULL_REVISION
+#       |
+#      rev1
+#       |
+#     rev2b
+#       |
+#     rev3b
+feature_branch = {'rev1': [NULL_REVISION],
+                  'rev2b': ['rev1'], 'rev3b': ['rev2b']}
+
+
+# History shortcut
+#  NULL_REVISION
+#       |
+#     rev1------
+#     /  \      \
+#  rev2a rev2b rev2c
+#    |  /   \   /
+#  rev3a    reveb
+history_shortcut = {'rev1': [NULL_REVISION], 'rev2a': ['rev1'],
+                    'rev2b': ['rev1'], 'rev2c': ['rev1'],
+                    'rev3a': ['rev2a', 'rev2b'], 'rev3b': ['rev2b', 'rev2c']}
+
+
+class TestGraphWalker(TestCaseWithMemoryTransport):
+
+    def make_graph(self, ancestors):
+        tree = self.prepare_memory_tree('.')
+        self.build_ancestry(tree, ancestors)
+        tree.unlock()
+        return tree.branch.repository.get_graph()
+
+    def prepare_memory_tree(self, location):
+        tree = self.make_branch_and_memory_tree(location)
+        tree.lock_write()
+        tree.add('.')
+        return tree
+
+    def build_ancestry(self, tree, ancestors):
+        """Create an ancestry as specified by a graph dict
+
+        :param tree: A tree to use
+        :param ancestors: a dict of {node: [node_parent, ...]}
+        """
+        pending = [NULL_REVISION]
+        descendants = {}
+        for descendant, parents in ancestors.iteritems():
+            for parent in parents:
+                descendants.setdefault(parent, []).append(descendant)
+        while len(pending) > 0:
+            cur_node = pending.pop()
+            for descendant in descendants.get(cur_node, []):
+                if tree.branch.repository.has_revision(descendant):
+                    continue
+                parents = [p for p in ancestors[descendant] if p is not
+                           NULL_REVISION]
+                if len([p for p in parents if not
+                    tree.branch.repository.has_revision(p)]) > 0:
+                    continue
+                tree.set_parent_ids(parents)
+                if len(parents) > 0:
+                    left_parent = parents[0]
+                else:
+                    left_parent = NULL_REVISION
+                tree.branch.set_last_revision_info(
+                    len(tree.branch._lefthand_history(left_parent)),
+                    left_parent)
+                tree.commit(descendant, rev_id=descendant)
+                pending.append(descendant)
+
+    def test_lca(self):
+        """Test finding least common ancestor.
+
+        ancestry_1 should always have a single common ancestor
+        """
+        graph = self.make_graph(ancestry_1)
+        self.assertEqual(set([NULL_REVISION]),
+                         graph.find_lca(NULL_REVISION, NULL_REVISION))
+        self.assertEqual(set([NULL_REVISION]),
+                         graph.find_lca(NULL_REVISION, 'rev1'))
+        self.assertEqual(set(['rev1']), graph.find_lca('rev1', 'rev1'))
+        self.assertEqual(set(['rev1']), graph.find_lca('rev2a', 'rev2b'))
+
+    def test_lca_criss_cross(self):
+        """Test least-common-ancestor after a criss-cross merge."""
+        graph = self.make_graph(criss_cross)
+        self.assertEqual(set(['rev2a', 'rev2b']),
+                         graph.find_lca('rev3a', 'rev3b'))
+        self.assertEqual(set(['rev2b']),
+                         graph.find_lca('rev3a', 'rev3b', 'rev2b'))
+
+    def test_lca_shortcut(self):
+        """Test least-common ancestor on this history shortcut"""
+        graph = self.make_graph(history_shortcut)
+        self.assertEqual(set(['rev2b']), graph.find_lca('rev3a', 'rev3b'))
+
+    def test_recursive_unique_lca(self):
+        """Test finding a unique least common ancestor.
+
+        ancestry_1 should always have a single common ancestor
+        """
+        graph = self.make_graph(ancestry_1)
+        self.assertEqual(NULL_REVISION,
+                         graph.find_unique_lca(NULL_REVISION, NULL_REVISION))
+        self.assertEqual(NULL_REVISION,
+                         graph.find_unique_lca(NULL_REVISION, 'rev1'))
+        self.assertEqual('rev1', graph.find_unique_lca('rev1', 'rev1'))
+        self.assertEqual('rev1', graph.find_unique_lca('rev2a', 'rev2b'))
+
+    def test_unique_lca_criss_cross(self):
+        """Ensure we don't pick non-unique lcas in a criss-cross"""
+        graph = self.make_graph(criss_cross)
+        self.assertEqual('rev1', graph.find_unique_lca('rev3a', 'rev3b'))
+
+    def test_unique_lca_null_revision(self):
+        """Ensure we pick NULL_REVISION when necessary"""
+        graph = self.make_graph(criss_cross2)
+        self.assertEqual('rev1b', graph.find_unique_lca('rev2a', 'rev1b'))
+        self.assertEqual(NULL_REVISION,
+                         graph.find_unique_lca('rev2a', 'rev2b'))
+
+    def test_unique_lca_null_revision2(self):
+        """Ensure we pick NULL_REVISION when necessary"""
+        graph = self.make_graph(ancestry_2)
+        self.assertEqual(NULL_REVISION,
+                         graph.find_unique_lca('rev4a', 'rev1b'))
+
+    def test_common_ancestor_two_repos(self):
+        """Ensure we do unique_lca using data from two repos"""
+        mainline_tree = self.prepare_memory_tree('mainline')
+        self.build_ancestry(mainline_tree, mainline)
+        mainline_tree.unlock()
+
+        # This is cheating, because the revisions in the graph are actually
+        # different revisions, despite having the same revision-id.
+        feature_tree = self.prepare_memory_tree('feature')
+        self.build_ancestry(feature_tree, feature_branch)
+        feature_tree.unlock()
+        graph = mainline_tree.branch.repository.get_graph(
+            feature_tree.branch.repository)
+        self.assertEqual('rev2b', graph.find_unique_lca('rev2a', 'rev3b'))
+
+    def test_graph_difference(self):
+        graph = self.make_graph(ancestry_1)
+        self.assertEqual((set(), set()), graph.find_difference('rev1', 'rev1'))
+        self.assertEqual((set(), set(['rev1'])),
+                         graph.find_difference(NULL_REVISION, 'rev1'))
+        self.assertEqual((set(['rev1']), set()),
+                         graph.find_difference('rev1', NULL_REVISION))
+        self.assertEqual((set(['rev2a', 'rev3']), set(['rev2b'])),
+                         graph.find_difference('rev3', 'rev2b'))
+        self.assertEqual((set(['rev4', 'rev3', 'rev2a']), set()),
+                         graph.find_difference('rev4', 'rev2b'))
+
+    def test_graph_difference_criss_cross(self):
+        graph = self.make_graph(criss_cross)
+        self.assertEqual((set(['rev3a']), set(['rev3b'])),
+                         graph.find_difference('rev3a', 'rev3b'))
+        self.assertEqual((set([]), set(['rev3b', 'rev2b'])),
+                         graph.find_difference('rev2a', 'rev3b'))
+
+    def test_stacked_parents_provider(self):
+
+        class ParentsProvider(object):
+
+            def __init__(self, ancestry):
+                self.ancestry = ancestry
+
+            def get_parents(self, revisions):
+                return [self.ancestry.get(r, None) for r in revisions]
+
+        parents1 = ParentsProvider({'rev2': ['rev3']})
+        parents2 = ParentsProvider({'rev1': ['rev4']})
+        stacked = graph._StackedParentsProvider([parents1, parents2])
+        self.assertEqual([['rev4',], ['rev3']],
+                         stacked.get_parents(['rev1', 'rev2']))
+        self.assertEqual([['rev3',], ['rev4']],
+                         stacked.get_parents(['rev2', 'rev1']))
+        self.assertEqual([['rev3',], ['rev3']],
+                         stacked.get_parents(['rev2', 'rev2']))
+        self.assertEqual([['rev4',], ['rev4']],
+                         stacked.get_parents(['rev1', 'rev1']))

=== modified file 'bzrlib/tests/test_revision.py'
--- bzrlib/tests/test_revision.py	2007-01-17 15:37:08 +0000
+++ bzrlib/tests/test_revision.py	2007-06-08 21:48:42 +0000
@@ -23,7 +23,7 @@
     )
 from bzrlib.branch import Branch
 from bzrlib.errors import NoSuchRevision
-from bzrlib.graph import Graph
+from bzrlib.deprecated_graph import Graph
 from bzrlib.revision import (find_present_ancestors, combined_graph,
                              common_ancestor,
                              is_ancestor, MultipleRevisionSources,

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2007-05-24 15:36:19 +0000
+++ bzrlib/workingtree.py	2007-06-08 21:48:42 +0000
@@ -2085,13 +2085,9 @@
             #       inventory and calls tree._write_inventory(). Ultimately we
             #       should be able to remove this extra flush.
             self.flush()
-            from bzrlib.revision import common_ancestor
-            try:
-                base_rev_id = common_ancestor(self.branch.last_revision(),
-                                              old_tip,
-                                              self.branch.repository)
-            except errors.NoCommonAncestor:
-                base_rev_id = None
+            graph = self.branch.repository.get_graph()
+            base_rev_id = graph.find_unique_lca(self.branch.last_revision(),
+                                                old_tip)
             base_tree = self.branch.repository.revision_tree(base_rev_id)
             other_tree = self.branch.repository.revision_tree(old_tip)
             result += merge.merge_inner(

